//
// This file is part of the GNU ARM Eclipse distribution.
// Copyright (c) 2014 Liviu Ionescu.
//

// ----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include "diag/Trace.h"

#include "Timer.h"
#include "sinetable.h"
#include "freqtable.h"

// ----------------------------------------------------------------------------
//
// Standalone STM32F4 led blink sample (trace via ITM).
//
// In debug configurations, demonstrate how to print a greeting message
// on the trace device. In release configurations the message is
// simply discarded.
//
// Then demonstrates how to blink a led with 1 Hz, using a
// continuous loop and SysTick delays.
//
// Trace support is enabled by adding the TRACE macro definition.
// By default the trace messages are forwarded to the ITM output,
// but can be rerouted to any device or completely suppressed, by
// changing the definitions required in system/src/diag/trace_impl.c
// (currently OS_USE_TRACE_ITM, OS_USE_TRACE_SEMIHOSTING_DEBUG/_STDOUT).
//


// ----- main() ---------------------------------------------------------------

// Sample pragmas to cope with warnings. Please note the related line at
// the end of this function, used to pop the compiler diagnostics status.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#pragma GCC diagnostic ignored "-Wmissing-declarations"
#pragma GCC diagnostic ignored "-Wreturn-type"

int32_t time, button_time;
TIM_HandleTypeDef TimHandle;
ADC_HandleTypeDef AdcHandle;
DAC_HandleTypeDef    DacHandle;

/* Variable used to get converted value */
__IO uint16_t adcBuffer[8];
uint16_t dacBuffer[32];
float inBuffer[32], outBuffer[32]; // interleaved - LRLRLRLRLRLRLRLRLRLRLR - inBuffer[frame << 1] inBuffer[(frame << 1) + 1]
float millisecs;
int32_t ledselected, adccount;
uint32_t seed;
float phase1, phase2;

// for ADC smoothing
uint32_t smoothadc[8], avgtotal[8];
uint32_t adc0average[64];
uint32_t adc1average[64];
int16_t avgindex;

void
led_init()
{
  // Enable GPIO Peripheral clock
  __HAL_RCC_GPIOD_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStructure;

  // Configure pin in output push/pull mode
  GPIO_InitStructure.Pin = GPIO_PIN_12|GPIO_PIN_13|GPIO_PIN_14|GPIO_PIN_15;
  GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStructure.Speed = GPIO_SPEED_FAST;
  GPIO_InitStructure.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOD, &GPIO_InitStructure);

}

void
switch_init()
{
  // Enable GPIO Peripheral clock
  __HAL_RCC_GPIOA_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStructure;

  // Configure pin for polling
  GPIO_InitStructure.Pin = GPIO_PIN_0;
  GPIO_InitStructure.Mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.Speed = GPIO_SPEED_FAST;
  GPIO_InitStructure.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStructure);


  // Configure for interrupt
  /* Configure PA0 pin as input floating */
/*  GPIO_InitStructure.Pin = GPIO_PIN_0;
  GPIO_InitStructure.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStructure.Pull = GPIO_NOPULL;
  GPIO_InitStructure.Speed = GPIO_SPEED_FAST;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStructure);*/

  /* Enable and set EXTI Line0 Interrupt to the lowest priority */
//  HAL_NVIC_SetPriority(EXTI0_IRQn, 2, 0);
//  HAL_NVIC_EnableIRQ(EXTI0_IRQn);

}

void
timer3_init()
{
	  __HAL_RCC_TIM3_CLK_ENABLE();

	  TimHandle.Instance = TIM3;
	  TimHandle.Init.Period = 8399; // Clock = 168000000/2 = 84000000/(8399+1) = 10000
	  TimHandle.Init.Prescaler = 0;
	  TimHandle.Init.ClockDivision = 0;
	  TimHandle.Init.CounterMode = TIM_COUNTERMODE_UP;
	  HAL_TIM_Base_Init(&TimHandle) ;
	  HAL_TIM_Base_Start_IT(&TimHandle);

	  /*##-2- Configure the NVIC for TIMx ########################################*/
	  /* Set Interrupt Group Priority */
	  HAL_NVIC_SetPriority(TIM3_IRQn, 4, 0);
	  HAL_NVIC_EnableIRQ(TIM3_IRQn);
}



int
main(int argc, char* argv[])
{
	timer_start();

	led_init();
	switch_init();
	timer3_init();

	// initialize globals
	time = 240;
	button_time = 0;
	millisecs = 0.0f;
	ledselected = 0;

	// Infinite loop
	while (1)
	{
		timer_sleep(100);
		if(HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_0))
			time = 80;
		else
			time = 240;
	}
  // Infinite loop, never return.
}

// interrupt from switch - pin A0
void EXTI0_IRQHandler(void)
{
	 HAL_GPIO_EXTI_IRQHandler(GPIO_PIN_0);
}

// callback from switch interrupt function HAL_GPIO_EXTI_IRQHandler()
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
// debouncing
	int32_t time_elapsed;
	time_elapsed = HAL_GetTick() - button_time;
	if(time_elapsed < 1)
		return;
//	time = time_elapsed;
	button_time = HAL_GetTick();
    if(time > 100)
    	time = 80;
    else
    	time = 240;
}

// interrupt from timer 3
void TIM3_IRQHandler(void)
{
  HAL_TIM_IRQHandler(&TimHandle);
}

// callback from timer interrupt function HAL_TIM_IRQHandler()
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
//	HAL_GPIO_TogglePin(GPIOD, GPIO_PIN_14);
	adccount = 0;
	millisecs += 0.1f;
	if(millisecs > time)
	{
		millisecs = 0.0f;
		switch(ledselected)
		{
		case 0:
			HAL_GPIO_WritePin(GPIOD, GPIO_PIN_12, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOD, GPIO_PIN_13, GPIO_PIN_SET);
			break;
		case 1:
			HAL_GPIO_WritePin(GPIOD, GPIO_PIN_13, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOD, GPIO_PIN_14, GPIO_PIN_SET);
			break;
		case 2:
			HAL_GPIO_WritePin(GPIOD, GPIO_PIN_14, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOD, GPIO_PIN_15, GPIO_PIN_SET);
			break;
		case 3:
			HAL_GPIO_WritePin(GPIOD, GPIO_PIN_15, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOD, GPIO_PIN_12, GPIO_PIN_SET);
			break;
		}
		ledselected += 1;
		ledselected &= 3;
	}
}

// interrupt from DMA 2 - Stream 0 (ADC1)
void DMA2_Stream0_IRQHandler(void)
{
  HAL_DMA_IRQHandler(AdcHandle.DMA_Handle);
}

// callback from DMA interrupt function HAL_DMA_IRQHandler()
void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef* AdcHandle)
{
  time = (adcBuffer[0] * adcBuffer[0]) >> 12;

  avgtotal[0] -= adc0average[avgindex];
  adc0average[avgindex] = adcBuffer[0];
  avgtotal[0] += adc0average[avgindex];
  smoothadc[0] = avgtotal[0] >> 6;

  avgtotal[1] -= adc1average[avgindex];
  adc1average[avgindex] = adcBuffer[1];
  avgtotal[1] += adc1average[avgindex];
  smoothadc[1] = avgtotal[1] >> 6;
  avgindex++;
  if(avgindex > 63) avgindex = 0;
}

void audioBlock(float *input, float *output, int32_t samples)
{
	int32_t iphase, i;
	float phaseinc1, phaseinc2, phase2mod;
	float knob0;

	knob0 = (float)(smoothadc[0]);

	phaseinc1 = sineinc[512];
	phaseinc2 = phaseinc1 * 1.5f;

	for(i = 0; i < samples; i++)
	{
		// phasor~ 1
		phase1 += phaseinc1;
		iphase = (int32_t)(phase1 * 8192.0f);	// might overflow if samples is too large.
//		output[i<<1] = sinetable[iphase & 0x00001fff];

		phase2 += phaseinc2;

		phase2mod = phase2 + (sinetable[iphase & 0x00001fff] * knob0 * 0.01f);
		iphase = (int32_t)(phase2mod * 8192.0f);	// might overflow if samples is too large.
		output[(i<<1) + 1] = sinetable[iphase & 0x00001fff];
	}
	while(phase1 >= 1.0f) phase1 -= 1.0f;
	while(phase1 < 0.0f) phase1 += 1.0f;

	while(phase2 >= 1.0f) phase2 -= 1.0f;
	while(phase2 < 0.0f) phase2 += 1.0f;

}
// interrupt from DMA 1 - Stream 5 (DAC Ch 1)
void DMA1_Stream5_IRQHandler(void)
{
  HAL_DMA_IRQHandler(DacHandle.DMA_Handle1);
}

// callback from DMA interrupt function HAL_DMA_IRQHandler()

void HAL_DAC_ConvCpltCallbackCh1(DAC_HandleTypeDef* DacHandle)
{
	int i;

	audioBlock(inBuffer, outBuffer, 16);
	for(i = 0; i < 16; i++)
	{
		dacBuffer[i+16] = (int16_t)((outBuffer[(i * 2)] + outBuffer[(i * 2)+1] + 2.0f) * 1023.0f);
	}
}

void HAL_DAC_ConvHalfCpltCallbackCh1(DAC_HandleTypeDef* DacHandle)
{
	int i,j;


	audioBlock(inBuffer, outBuffer, 16);
	for(i = 0, j = 0; i < 16; i++, j+=2)
	{
		dacBuffer[i] = (int16_t)((outBuffer[j] + outBuffer[j+1] + 2.0f) * 1023.0f);
	}
}

/**
* @brief  This function handles DMA interrupt request.
* @param  None
* @retval None
*/
void DACx_DMA_IRQHandler2(void)
{
  HAL_DMA_IRQHandler(DacHandle.DMA_Handle2);
}

#pragma GCC diagnostic pop

// ----------------------------------------------------------------------------
